#!/bin/bash
# Copyright 2018 Michal Žejdl
#
# open the named pipe for reading by mysql process
# which is used by other golems for modifying the database (insert, update)
#
# mysql binary with named pipe doesn't wait for server and doesn't reconnect
# so here is a workaround (no time to write own mysql pipe in C)

# dependencies
for program in mysql pidof wc cut tail grep date kill
do
	which "$program" >/dev/null || exit
done

# config
confdir=$(dirname $0)
config=$confdir/defaults.sh
test -f $config || exit
source $config

sem="$rundir/$(basename $0).sem"

# retry [seconds]
reconnect=10
# search for this string to check if mysql is disconnected from a server
# cite: For global errors, the SQLSTATE value is always 'HY000' (general error)
# https://dev.mysql.com/doc/refman/5.5/en/global-error-reference.html
search="^ERROR.*HY000"

pipe="$mysqlwpipe"
errlog="$logdir/mysql.err"

# the wait interuptible by removing the "šém" file for this process from outside
function idle
{
	test $# -eq 1 || return

	local delay=$1
	local timer=0

	while test $((timer++)) -lt "$delay"
	do
		test -e "$sem" || exit
		sleep 1
	done
}

# checks pid if there is such process with the same given name
# there can always be another process with the same pid and name
# when our former process is terminated early but this is my best approach
function pid_has_name
{
	# should return 1
	test $# -eq 2 || return

	local pid=$1
	local name=$2
	local process

	for process in $(pidof "$name")
	do
		# should return 0
		test "$process" = "$pid" && return
	done

	return 1
}


while true
do
	# repeat until job mysql is running
	# and the mysql process is started (see below)
	until jobs %mysql
	do
		# bash runs the mysql job (&)
		# and wait until some process opens the pipe for writing
		# (exec 3>"$pipe" here or in the other scripts)
		# after that the mysql binary is started
		# if connect failed, mysql exits despite the -w setting
		#
		# if open for writing precede the mysql command
		# in the very same script
		# then mysql inherits this file descriptor and wouldn't stop
		# when others file descriptors (opened for writing) would close
		# (there is always less reliable option
		# to send quit command to pipe)
		mysql -fnNsw --reconnect $modify_opts <"$pipe" \
			>>"$logdir/mysql.out" 2>>"$errlog" \
			& mysql_pid=$!

		echo "$mysql_pid" >"$rundir/mysql.pid"

		# while previously started job is still running
		while jobs %mysql
		do
			# check if the mysql process is started
			# (by other nana process
			# which opened the pipe for writing)
			pid_has_name "$mysql_pid" "mysql" && break

			idle "$reconnect"
		done

		# if mysql failed to connect
		# the job is gone and loop will continue
		# else we are here because the pid has its process
		# so the loop ends here
	done


	# mysql should be connected now
	# let's check repeatedly if it's still true

	# pointer to the end of file
	last_known_line=$(wc -l "$errlog" | cut -d" " -f1)

	while true
	do
		last_checked_line="$last_known_line"
		last_known_line=$(wc -l "$errlog" | cut -d" " -f1)

		#echo "last_checked_line $last_checked_line"
		#echo "last_known_line   $last_known_line"

		if tail -n+$((last_checked_line + 1)) "$errlog" \
			| grep -q "$search"
		then
			echo -n "$(date '+%F %T') mysql is disconnected"
			echo " from a server"

			# send the command connect to mysql
			# causes mysql exit if server is down
			# (Can't connect to MySQL server)
			# or reconnect if server is up
			#
			# it should be simplier to kill the mysql job
			# and start over
			kill %mysql
			# job control is not clear to me (set -b?)
			# so rather to wait for mysql to end
			while jobs %mysql
			do
				idle 1
			done

			break
		fi

		idle "$reconnect"
	done
done
